/**
 * This file is part of the JustKnit package.
 *
 * @copyright JustKnit (Iris Hammer) <iris@justknit.de>
 * @license All rights reserved (@https://dejure.org/gesetze/UrhG/69a.html)
 * Created on 03-20-2023
 *
 * @package justknit
 */
const KnApKey = 'knitApp';
const KnApDfVal = {
            sid: 0,
            sessionId: 0,
//            lastDisplayTime: 0,
//            returningVisitor: !1,
//            displayCount: 0,
//            dismissed: !1,
//            added: !1,
            sessions: 0,
//            nextSession: 0,
            theme: 'light'
};

/**
 * BEGIN Nachtmodus
 * Alles was mit dem Nachtmodus zu tun hat.
 *
 **/
const themeDisplay = document.getElementById("darkSwitchLabel");
const darkSwitch = document.getElementById("darkSwitch");
function getKnitApp() {
//   var KnitApp = (localStorage.getItem(KnApKey) !== null);
   (localStorage.getItem(KnApKey) !== null) ? KnitApp = JSON.parse(localStorage.getItem(KnApKey)) : KnitApp = KnApDfVal;
    return KnitApp;
//    JSON.parse(localStorage.getItem(KnApKey)) || KnApDfVal;
}
function updateKnitApp(kaVal) {
    localStorage.setItem(KnApKey, JSON.stringify(kaVal));
}
function initTheme() {
    var kaVal = getKnitApp();
    var darkThemeSelected = (kaVal !== null && kaVal.theme === 'dark');
    // update checkbox
    darkSwitch ? darkSwitch.checked = darkThemeSelected : null;
    (document.documentElement.dataset.bsTheme == 'dark') ? kaVal.theme = 'dark' : kaVal.theme = 'light';
//    darkThemeSelected ? document.documentElement.dataset.appliedMode = 'dark' : document.documentElement.dataset.appliedMode = 'light';
    darkThemeSelected ? document.documentElement.dataset.bsTheme = 'dark' : document.documentElement.dataset.bsTheme = 'light';
    updateKnitApp(kaVal);
}
setTimeout(() => {
    if(darkSwitch) {
        initTheme(); // on page load, if user has already selected a specific theme -> apply it
        /** TODO: themeDisplay muss dann noch mit dem lexiconstring gefüllt werden. **/
        darkSwitch.addEventListener('change', function(event){
            changeTheme();
        });
        function changeTheme() {
            var kaVal = getKnitApp();
            if(darkSwitch.checked) { // dark theme has been selected
//                document.documentElement.dataset.appliedMode = 'dark';
                document.documentElement.dataset.bsTheme = 'dark';
                kaVal.theme = 'dark';
            } else {
//                document.documentElement.dataset.appliedMode = 'light';
                document.documentElement.dataset.bsTheme = 'light';
                kaVal.theme = 'light';
            }
            updateKnitApp(kaVal);
        };
    }
}, 1000);
/** END Nachtmodus **/
/**
 * BEGIN Offline Modus
 * Show Banner to hint User PWA not usable offline
 **/
window.addEventListener('offline', () => {
    console.log('offline');
    //showOfflineBanner();
});
window.addEventListener('online', () => {
    console.log('online');
    //showOnlineBanner();
});
/** END Nachtmodus **/

/**
 * BEGIN ServiceWorker for PWA
 * New Version to use a ServiceWorker to provide a PWA
 **/
//console.log('isLoggedIn', res.isLoggedIn);
console.log('project', res.project);
const noCacheUrl1 = ['/rechnen/', '/projekte/', 'passwort', 'login'];
if (noCacheUrl1.some(item => document.location.href.includes(item))) console.log('url includes ',  document.location.href );
/** TODO
 * Hier muss dann zukünftig nicht das login geprüft werden
 * Sondern:
 *  - Beim Besuch der DankeSeite werden Werte in localStorage @json 'knitApp' gespeichert
 *    z. B. die sid, das aktuelle neue Produkt, die subscriptionend
 *  - Wenn dann die sid gesetzt ist und sich der user das erste Mal einlogged, wird
 *    der serviceWorker geladen.
 *  - Wenn der User sich einlogged und die sid noch nicht gesetzt ist, muss die @json 'knitApp'
 *    gesetzt werden mit allen weiteren Daten
 *
 */

if (res.isLoggedIn) {
  console.log('isLoggedIn', res.isLoggedIn);
  if (('serviceWorker' in navigator)) {
    console.log('PWA: SW supported I001');
    navigator.serviceWorker.register('sw.js',{scope: "./"})
    .then(function(reg) {
      // Registration was successful
      console.log('PWA: SW registration successful with scope: I003 ', reg.scope);
      if (reg.installing) {
        console.log('PWA: SW installing I004');
      } else if (reg.waiting) {
        console.log('PWA: SW waiting');
      } else if (reg.active) {
        console.log('PWA: SW active');
      }

    })
    .then(function(reg) {
      console.log('PWA: SW ready');
      return navigator.serviceWorker.ready;
      console.log('PWA: SW :', reg);
    })
    .then(function(reg) {
      reg.addEventListener('updatefound', function () {
        console.log('PWA: SW updatefound');
        //installingWorker.state;
        const installingWorker = reg.installing;
        installingWorker.onstatechange = () => {
          if (installingWorker.state === 'installed' &&  navigator.serviceWorker.controller) {
            console.log('PWA: SW Show UpdateToast??');
            installingWorker.postMessage({action: 'skipWaiting'});
            // hier sollte dann vllt der cache gelöscht werden?
            //showUpdateToast();
            //installingWorker.postMessage({action: 'skipWaiting'});
          }
        }
      })
    })
    .catch(function(err) {
      // registration failed :(
      console.log('PWA: SW registration failed: ', err);
    });
  } else {
    console.log('PWA: SW NOT supported');
  }
} else {
  console.log('your not logged-in', res.isLoggedIn);
}

/*
navigator.serviceWorker.register('/sw.js').then(reg => {
  reg.installing; // the installing worker, or undefined
  reg.waiting; // the waiting worker, or undefined
  reg.active; // the active worker, or undefined

  reg.addEventListener('updatefound', () => {
    // A wild service worker has appeared in reg.installing!
    const newWorker = reg.installing;

    newWorker.state;
    // "installing" - the install event has fired, but not yet complete
    // "installed"  - install complete
    // "activating" - the activate event has fired, but not yet complete
    // "activated"  - fully active
    // "redundant"  - discarded. Either failed install, or it's been
    //                replaced by a newer version

    newWorker.addEventListener('statechange', () => {
      // newWorker.state has changed
    });
  });
});

navigator.serviceWorker.addEventListener('controllerchange', () => {
  // This fires when the service worker controlling this page
  // changes, eg a new worker has skipped waiting and become
  // the new active worker.
});

*/
// ODER
    // .then((registration) => { console.log("Service worker registration succeeded:", registration);
    //   },
    // (error) => { console.error(`Service worker registration failed: ${error}`);
    //   },
    // ODER
    // .then(registration => resolve(registration))
    // .catch(fehler => reject(fehler))
/*
if ('serviceWorker' in navigator) {
  navigator.serviceWorker.register('/sw.js').then(function(registration) {
    // Registration was successful
    console.log('ServiceWorker registration successful with scope: ',    registration.scope);
  }).catch(function(err) {
    // registration failed :(
    console.log('ServiceWorker registration failed: ', err);
  });
}

*/
/* Oder:
  if ('serviceWorker' in navigator) {
    try {
      const registration = await navigator.serviceWorker.register(
        'sw.js',
        {
          scope: './',
        }
      );
      if (registration.installing) {
        console.log('Service worker installing');
      } else if (registration.waiting) {
        console.log('Service worker installed');
      } else if (registration.active) {
        console.log('Service worker active');
      }
    } catch (error) {
      console.error(`Registration failed with ${error}`);
    }
  }
registerServiceWorker();
*/



/*
if ("serviceWorker" in navigator) {
  navigator.serviceWorker
    .register("service-worker.js", {
      scope: "./",
    })
    .then((registration) => {
      let serviceWorker;
      if (registration.installing) {
        serviceWorker = registration.installing;
        document.querySelector("#kind").textContent = "installing";
      } else if (registration.waiting) {
        serviceWorker = registration.waiting;
        document.querySelector("#kind").textContent = "waiting";
      } else if (registration.active) {
        serviceWorker = registration.active;
        document.querySelector("#kind").textContent = "active";
      }
      if (serviceWorker) {
        // logState(serviceWorker.state);
        serviceWorker.addEventListener("statechange", (e) => {
          // logState(e.target.state);
        });
      }
    })
    .catch((error) => {
      // Something went wrong during registration. The service-worker.js file
      // might be unavailable or contain a syntax error.
    });
} else {
  // The current browser doesn't support service workers.
  // Perhaps it is too old or we are not in a Secure Context.
}
 *
 */
/** END ServiceWorker for PWA **/





