/**
 * This file is part of the JustKnit package.
 *
 * @copyright JustKnit (Iris Hammer) <iris@justknit.de>
 * @license All rights reserved (@https://dejure.org/gesetze/UrhG/69a.html)
 * Created on 2024-06-27
 *
 * @package justknit
 */
/**
 * ServiceWorker for JustKnit PWA
 * Installable Webapp to calculate yout knitting Projects
 * @var string $cacheName named local Cache Storage of PWA
 * @var array  $staticRes list of all static resources needed for PWA
 *
 */
    const version = '0.001_'+[[*editedon:strtotime]];
    const cacheName = 'jk_' + version;
    const offlineRes = [
        '[[~[[GetID? &alias=`rechnen`]]]]'
        ,'[[~[[GetID? &alias=`offline`]]]]'
        ,'[[~[[GetID? &alias=`justknit`]]]]'
        ,'[[~[[GetID? &alias=`manifest`]]]]'
        ,'assets/components/justknit/css/bootstrap.min.css'
        ,'assets/components/justknit/css/bootstrap-icons.css'
        ,'assets/components/justknit/css/googlefonts.css'
        ,'assets/components/justknit/css/justknit.basis.css'
        ,'assets/components/justknit/js/web/popper.min.js'
        ,'assets/components/justknit/js/web/bootstrap.min.js'
        ,'assets/components/justknit/js/web/justknit.js'
        ,'assets/components/justknit/fonts/bootstrap-icons/bootstrap-icons.woff2'

    ];
    const offlineUrl = '[[~[[GetID? &alias=`offline`]]]]';
    const noCacheUrl = ['/rechnen/', '/projekte/', 'passwort', 'login'];
    // add Hash to ServiceWorker to force update on changed resources
    const resHash = [
        [[!pdoMenu?
            &parents=`0`
            &context=`justknit`
            &showHidden=`0`
            &useWeblinkUrl=`0`
            &tplOuter=`@INLINE [[+wrapper]]`
            &tpl=`@INLINE '[[+editedon]]',[[+wrapper]]`
        ]], '[[*editedon:strtotime]]'
    ];

//edited:  [[*editedon:strtotime]]: [[*editedon]]

self.addEventListener('install', function(event) {
    console.log('PWA: SW e.install I002');
    self.skipWaiting();
    /**
     * Hier soll beim ersten install alles was angefragt wird in den cache geladen werden.
     * Und dazu soll irgendwo ein hash gespeichert werden, damit ich weiß wann ich aktualisiere ??
     * Nein. Das ist Quatsch. Ich aktualisiere einfach zeitgesteuert. Auf Basis von der Aktivierung vom Account
     * Das geht so nicht, weil beim install nicht gefetcht wird.
     * Ich muss die Sachen laden, die nach dem Login erforderlich sind.
    */
    event.waitUntil(
        caches.open(cacheName)
        .then(function(cache) {
            cache.addAll(offlineRes);
        })
        .catch(function(err) {
            console.log('PWA: ?? ERR: ', err);
        })
    );
/*
    event.waitUntil((async function () {
      const cache = await caches.open(cacheName);
      await cache.addAll(offlineRes);
    })(),
  );
*/
});

self.addEventListener('activate', function(event) {
    console.log('PWA: SW e.activate I005');
    event.waitUntil(
        (async () => {
        const cacheRes = await caches.keys();
        await Promise.all(cacheRes.map(async (cacheName) => {
        if (self.cacheName !== cacheName) {
            await caches.delete(cacheName);
        }
        }));
    })()

    );
});

/*
// remove old cache if any
self.addEventListener('activate', (event) => {
  event.waitUntil((async () => {
    const cacheNames = await caches.keys();

    await Promise.all(cacheNames.map(async (cacheName) => {
      if (self.cacheName !== cacheName) {
        await caches.delete(cacheName);
      }
    }));
  })());
});
*/
/**
 * ATTENTION
 * Hier beim 'fetch' muss eine genaue Strategie für die Verarbeitung der Responses erfolgen
 *  - Was wird im cache abgelegt?
 *  - Als erstes muss über den localStorage die Gültigkeit des Accounts geprüft werden.
 *  - eigentlich kann ich alles cachen. Nur muss
 */

self.addEventListener('fetch', function(event) {
    //console.log('PWA: SW e.fetch',event.request.url);
    //const url = new URL(event.request.url);

    event.respondWith(
        caches.open(cacheName)
        .then(async function(cache) {
            const cachedResponse = await cache.match(event.request);
            if (cachedResponse) return cachedResponse;
            const networkResponse = await fetch(event.request);
            // prevent noCacheUrl pages from being cached
//            if (noCacheUrl.some(item => event.request.url.includes(item))) return networkResponse;
//            console.log('fetch',event.request.url );
            console.log('Wer bin ich?? ', '[[*editedon]]');
            if (event.request.url.includes('/rechnen/') || event.request.url.includes('/projekte/') || event.request.url.includes('passwort') || event.request.url.includes('login')) {
//            if (noCacheUrl.some(item => event.request.url.includes(item))) {
                console.log('fetch:return networkResponse ',event.request.url );
                return networkResponse;
            }

            event.waitUntil(cache.put(event.request, networkResponse.clone()));
            console.log('fetch: cache.put',event.request.url );
            return networkResponse;
        })
/*        .then(async function(cache) {

            if (event.request.url.includes('login')) {
                console.log('fetch: login wurde angefordert',event.request.url );
            }
        })
*/
        .catch(function(err) {
            console.log('PWA: ?? ERR: ', err);
            return caches.match(offlineUrl);
        })
    );
});
/**
 * TODO: Keine Ahnung, was ich überhaupt will. Eigentlich soll nichts aus dem cache geholt werden.
 * Dann muss aber ein Hinweis da sein, dass der User offline ist, und die PWA offline nicht funktioniert.
 *
 *
 */
// Network only:
/*
self.addEventListener('fetch', function (event) {
    event.respondWith(
        fetch(event.request).then(function(networkResponse) {
            return networkResponse
        })
    )
})
*/
// Network first:
/*
self.addEventListener(’fetch’, function (event) {
    event.respondWith(
        fetch(event.request).catch(function() {
            return caches.match(event.request)
        })
    )
})
*/

self.addEventListener('sync', function(event) {
    console.log('PWA: SW e.sync');
});

self.addEventListener('push', function(event) {
    console.log('PWA: SW e.push');
});

/* Des hat funktioniert:
 * Des hat schon funktioniert. Aber ich kann den ServiceWorker nicht im Einzelschritt debuggen!!
 *
 */
/*  event.respondWith(
    caches.match(event.request)
      .then(function(response) {
        // Cache hit - return response
        if (response) {
          return response;
        }
        return fetch(event.request);
      }
    )
  );
*/

/*
self.addEventListener('fetch', function(event) {
    console.log('PWA: SW e.fetch');
    event.respondWith(
        caches.match(event.request)
        .then(function(cache) => {
            return cache || fetch(event.request)
        })
        .catch(function(err) {
            console.log('PWA: ?? ERR: ', err);
        })
    );

});

*/
/*
    event.respondWith(
    (async function () {
      const cache = await caches.open(cacheName);
      const cachedResponse = await cache.match(event.request);
      if (cachedResponse) return cachedResponse;
      const networkResponse = await fetch(event.request);
      event.waitUntil(cache.put(event.request, networkResponse.clone()));
      return networkResponse;
    })(),
  );
*/
/*    event.respondWith(
        caches.open(cacheName)
        .then(function(cache) {
            const cachedResponse = cache.match(event.request);
            if (cachedResponse) return cachedResponse;
            const networkResponse = fetch(event.request);
            event.waitUntil(cache.put(event.request, networkResponse.clone()));
            return networkResponse;
        })
        .catch(function(err) {
            console.log('PWA: ?? ERR: ', err);
        })
    )
*/
/*    event.respondWith(
        caches.open(cacheName)
        .then(async function(cache) {
            const cachedResponse = await cache.match(event.request);
            if (cachedResponse) return cachedResponse;
            const networkResponse = await fetch(event.request);
            event.waitUntil(cache.put(event.request, networkResponse.clone()));
            return networkResponse;
        })
        .catch(function(err) {
            console.log('PWA: ?? ERR: ', err);
        })
    )
*/

/*        (
        async function () {
      const cache = await caches.open(cacheName);
      const cachedResponse = await cache.match(event.request);
      if (cachedResponse) return cachedResponse;
      const networkResponse = await fetch(event.request);
      event.waitUntil(cache.put(event.request, networkResponse.clone()));
      return networkResponse;
        })(),
*/



/*
active:             ServiceWorker {scriptURL: 'https://justknit.now-dns.org/revolution/sw.js', state: 'activated', onstatechange: null, onerror: null}
backgroundFetch:    BackgroundFetchManager {}
cookies:            CookieStoreManager {}
installing:         null
navigationPreload:  NavigationPreloadManager {}
onupdatefound:      null
paymentManager:     PaymentManager {userHint: ''}
periodicSync:       PeriodicSyncManager {}
pushManager:        PushManager {}
scope:              "https://justknit.now-dns.org/revolution/"
sync:               SyncManager {}
updateViaCache:     "imports"
waiting:            null


*/
